@@ -1,3 +1,6 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
+
 #ifndef CEPH_CLIENT_INODE_H
 #define CEPH_CLIENT_INODE_H
 
@@ -10,10 +13,10 @@
 #include "osdc/ObjectCacher.h"
 #include "include/assert.h"
 
-class MetaSession;
+struct MetaSession;
 class Dentry;
 class Dir;
-class SnapRealm;
+struct SnapRealm;
 class Inode;
 
 struct Cap {
@@ -31,6 +34,8 @@ struct Cap {
 
   Cap() : session(NULL), inode(NULL), cap_item(this), cap_id(0), issued(0),
 	       implemented(0), wanted(0), seq(0), issue_seq(0), mseq(0), gen(0) {}
+
+  void dump(Formatter *f) const;
 };
 
 struct CapSnap {
@@ -58,6 +63,8 @@ struct CapSnap {
       writing(false), dirty_data(false), flush_tid(0),
       flushing_item(this)
   {}
+
+  void dump(Formatter *f) const;
 };
 
 
@@ -104,10 +111,21 @@ class Inode {
   version_t version;           // auth only
   version_t xattr_version;
 
+  // inline data
+  version_t  inline_version;
+  bufferlist inline_data;
+
   bool is_symlink() const { return (mode & S_IFMT) == S_IFLNK; }
   bool is_dir()     const { return (mode & S_IFMT) == S_IFDIR; }
   bool is_file()    const { return (mode & S_IFMT) == S_IFREG; }
 
+  bool has_dir_layout() const {
+    for (unsigned c = 0; c < sizeof(layout); c++)
+      if (*((const char *)&layout + c))
+	return true;
+    return false;
+  }
+
   unsigned flags;
 
   // about the dir (if this is one!)
@@ -167,13 +185,15 @@ class Inode {
     lsubdout(cct, mds, 15) << "inode.get on " << this << " " <<  ino << '.' << snapid
 		   << " now " << _ref << dendl;
   }
-  int put(int n=1) { 
+  /// private method to put a reference; see Client::put_inode()
+  int _put(int n=1) {
     _ref -= n; 
     lsubdout(cct, mds, 15) << "inode.put on " << this << " " << ino << '.' << snapid
 		   << " now " << _ref << dendl;
     assert(_ref >= 0);
     return _ref;
   }
+
   int get_num_ref() {
     return _ref;
   }
@@ -191,6 +211,7 @@ class Inode {
       rdev(0), mode(0), uid(0), gid(0), nlink(0),
       size(0), truncate_seq(1), truncate_size(-1),
       time_warp_seq(0), max_size(0), version(0), xattr_version(0),
+      inline_version(0),
       flags(0),
       dir_hashed(false), dir_replicated(false), auth_cap(NULL),
       dirty_caps(0), flushing_caps(0), flushing_cap_seq(0), shared_gen(0), cache_gen(0),
@@ -211,6 +232,16 @@ class Inode {
 
   vinodeno_t vino() { return vinodeno_t(ino, snapid); }
 
+  struct Compare {
+    bool operator() (Inode* const & left, Inode* const & right) {
+      if (left->ino.val < right->ino.val) {
+	return (left->snapid.val < right->snapid.val);
+      }
+      return false;
+    }
+  };
+
+  bool check_mode(uid_t uid, gid_t gid, gid_t *sgids, int sgid_count, uint32_t flags);
 
   // CAPS --------
   void get_open_ref(int mode);
@@ -231,6 +262,8 @@ class Inode {
 
   bool have_valid_size();
   Dir *open_dir();
+
+  void dump(Formatter *f) const;
 };
 
 ostream& operator<<(ostream &out, Inode &in);
