@@ -57,13 +57,13 @@ public:
   string readdir_start;  // starting _after_ this name
   uint64_t readdir_offset;
 
+  frag_t readdir_reply_frag;
   vector<pair<string,Inode*> > readdir_result;
   bool readdir_end;
   int readdir_num;
   string readdir_last_name;
 
   //possible responses
-  bool got_safe;
   bool got_unsafe;
 
   xlist<MetaRequest*>::item item;
@@ -90,7 +90,7 @@ public:
     ref(1), reply(0), 
     kick(false),
     readdir_offset(0), readdir_end(false), readdir_num(0),
-    got_safe(false), got_unsafe(false), item(this), unsafe_item(this),
+    got_unsafe(false), item(this), unsafe_item(this),
     lock("MetaRequest lock"),
     caller_cond(0), dispatch_cond(0),
     target(0) {
@@ -101,10 +101,25 @@ public:
 
   void set_inode(Inode *in);
   Inode *inode();
+  Inode *take_inode() {
+    Inode *i = _inode;
+    _inode = 0;
+    return i;
+  }
   void set_old_inode(Inode *in);
   Inode *old_inode();
+  Inode *take_old_inode() {
+    Inode *i = _old_inode;
+    _old_inode = NULL;
+    return i;
+  }
   void set_other_inode(Inode *in);
   Inode *other_inode();
+  Inode *take_other_inode() {
+    Inode *i = _other_inode;
+    _other_inode = 0;
+    return i;
+  }
   void set_dentry(Dentry *d);
   Dentry *dentry();
   void set_old_dentry(Dentry *d);
@@ -115,10 +130,14 @@ public:
     return this;
   }
 
-  void put() {
+  /// psuedo-private put method; use Client::put_request()
+  void _put() {
     if (--ref == 0)
       delete this;
   }
+  int get_num_ref() {
+    return ref;
+  }
 
   // normal fields
   void set_tid(tid_t t) { tid = t; }
