@@ -11,13 +11,13 @@
  * Foundation.  See file COPYING.
  * 
  */
-
 #ifndef CEPH_ENCODING_H
 #define CEPH_ENCODING_H
 
-#include <tr1/memory>
+#include "include/int_types.h"
+
+#include "include/memory.h"
 
-#include "inttypes.h"
 #include "byteorder.h"
 #include "buffer.h"
 #include "assert.h"
@@ -198,13 +198,13 @@ template<class A>
 inline void encode_array_nohead(const A a[], int n, bufferlist &bl)
 {
   for (int i=0; i<n; i++)
-    encode(a[n], bl);
+    encode(a[i], bl);
 }
 template<class A>
 inline void decode_array_nohead(A a[], int n, bufferlist::iterator &p)
 {
   for (int i=0; i<n; i++)
-    decode(a[n], p);
+    decode(a[i], p);
 }
 
 
@@ -286,15 +286,38 @@ inline void decode(T &o, bufferlist& bl)
 #include <deque>
 #include <vector>
 #include <string>
+#include <boost/optional.hpp>
 
 #ifndef _BACKWARD_BACKWARD_WARNING_H
 #define _BACKWARD_BACKWARD_WARNING_H   // make gcc 4.3 shut up about hash_*
 #endif
-#include <ext/hash_map>
-#include <ext/hash_set>
+#include "include/unordered_map.h"
+#include "include/unordered_set.h"
 
 #include "triple.h"
 
+// boost optional
+template<typename T>
+inline void encode(const boost::optional<T> &p, bufferlist &bl)
+{
+  __u8 present = static_cast<bool>(p);
+  ::encode(present, bl);
+  if (p)
+    ::encode(p.get(), bl);
+}
+
+template<typename T>
+inline void decode(boost::optional<T> &p, bufferlist::iterator &bp)
+{
+  __u8 present;
+  ::decode(present, bp);
+  if (present) {
+    T t;
+    p = t;
+    ::decode(p.get(), bp);
+  }
+}
+
 // pair
 template<class A, class B>
 inline void encode(const std::pair<A,B> &p, bufferlist &bl)
@@ -363,14 +386,14 @@ inline void decode(std::list<T>& ls, bufferlist::iterator& p)
 }
 
 template<class T>
-inline void encode(const std::list<std::tr1::shared_ptr<T> >& ls, bufferlist& bl)
+inline void encode(const std::list<ceph::shared_ptr<T> >& ls, bufferlist& bl)
 {
   // should i pre- or post- count?
   if (!ls.empty()) {
     unsigned pos = bl.length();
     unsigned n = 0;
     encode(n, bl);
-    for (typename std::list<std::tr1::shared_ptr<T> >::const_iterator p = ls.begin(); p != ls.end(); ++p) {
+    for (typename std::list<ceph::shared_ptr<T> >::const_iterator p = ls.begin(); p != ls.end(); ++p) {
       n++;
       encode(**p, bl);
     }
@@ -380,18 +403,18 @@ inline void encode(const std::list<std::tr1::shared_ptr<T> >& ls, bufferlist& bl
   } else {
     __u32 n = ls.size();    // FIXME: this is slow on a list.
     encode(n, bl);
-    for (typename std::list<std::tr1::shared_ptr<T> >::const_iterator p = ls.begin(); p != ls.end(); ++p)
+    for (typename std::list<ceph::shared_ptr<T> >::const_iterator p = ls.begin(); p != ls.end(); ++p)
       encode(**p, bl);
   }
 }
 template<class T>
-inline void decode(std::list<std::tr1::shared_ptr<T> >& ls, bufferlist::iterator& p)
+inline void decode(std::list<ceph::shared_ptr<T> >& ls, bufferlist::iterator& p)
 {
   __u32 n;
   decode(n, p);
   ls.clear();
   while (n--) {
-    std::tr1::shared_ptr<T> v(new T);
+    ceph::shared_ptr<T> v(new T);
     decode(*v, p);
     ls.push_back(v);
   }
@@ -481,18 +504,18 @@ inline void decode_nohead(int len, std::vector<T>& v, bufferlist::iterator& p)
 
 // vector (shared_ptr)
 template<class T>
-inline void encode(const std::vector<std::tr1::shared_ptr<T> >& v, bufferlist& bl)
+inline void encode(const std::vector<ceph::shared_ptr<T> >& v, bufferlist& bl)
 {
   __u32 n = v.size();
   encode(n, bl);
-  for (typename std::vector<std::tr1::shared_ptr<T> >::const_iterator p = v.begin(); p != v.end(); ++p)
+  for (typename std::vector<ceph::shared_ptr<T> >::const_iterator p = v.begin(); p != v.end(); ++p)
     if (*p)
       encode(**p, bl);
     else
       encode(T(), bl);
 }
 template<class T>
-inline void decode(std::vector<std::tr1::shared_ptr<T> >& v, bufferlist::iterator& p)
+inline void decode(std::vector<ceph::shared_ptr<T> >& v, bufferlist::iterator& p)
 {
   __u32 n;
   decode(n, p);
@@ -562,6 +585,17 @@ inline void decode(std::map<T,U>& m, bufferlist::iterator& p)
   }
 }
 template<class T, class U>
+inline void decode_noclear(std::map<T,U>& m, bufferlist::iterator& p)
+{
+  __u32 n;
+  decode(n, p);
+  while (n--) {
+    T k;
+    decode(k, p);
+    decode(m[k], p);
+  }
+}
+template<class T, class U>
 inline void encode_nohead(const std::map<T,U>& m, bufferlist& bl)
 {
   for (typename std::map<T,U>::const_iterator p = m.begin(); p != m.end(); ++p) {
@@ -613,19 +647,19 @@ inline void decode(std::multimap<T,U>& m, bufferlist::iterator& p)
   }
 }
 
-// hash_map
+// ceph::unordered_map
 template<class T, class U>
-inline void encode(const __gnu_cxx::hash_map<T,U>& m, bufferlist& bl)
+inline void encode(const unordered_map<T,U>& m, bufferlist& bl)
 {
   __u32 n = m.size();
   encode(n, bl);
-  for (typename __gnu_cxx::hash_map<T,U>::const_iterator p = m.begin(); p != m.end(); ++p) {
+  for (typename unordered_map<T,U>::const_iterator p = m.begin(); p != m.end(); ++p) {
     encode(p->first, bl);
     encode(p->second, bl);
   }
 }
 template<class T, class U>
-inline void decode(__gnu_cxx::hash_map<T,U>& m, bufferlist::iterator& p)
+inline void decode(unordered_map<T,U>& m, bufferlist::iterator& p)
 {
   __u32 n;
   decode(n, p);
@@ -637,17 +671,17 @@ inline void decode(__gnu_cxx::hash_map<T,U>& m, bufferlist::iterator& p)
   }
 }
 
-// hash_set
+// ceph::unordered_set
 template<class T>
-inline void encode(const __gnu_cxx::hash_set<T>& m, bufferlist& bl)
+inline void encode(const ceph::unordered_set<T>& m, bufferlist& bl)
 {
   __u32 n = m.size();
   encode(n, bl);
-  for (typename __gnu_cxx::hash_set<T>::const_iterator p = m.begin(); p != m.end(); ++p)
+  for (typename ceph::unordered_set<T>::const_iterator p = m.begin(); p != m.end(); ++p)
     encode(*p, bl);
 }
 template<class T>
-inline void decode(__gnu_cxx::hash_set<T>& m, bufferlist::iterator& p)
+inline void decode(ceph::unordered_set<T>& m, bufferlist::iterator& p)
 {
   __u32 n;
   decode(n, p);
@@ -697,6 +731,8 @@ inline void decode(std::deque<T>& ls, bufferlist::iterator& p)
   __u8 struct_v = v, struct_compat = compat;		     \
   ::encode(struct_v, bl);				     \
   ::encode(struct_compat, bl);				     \
+  buffer::list::iterator struct_compat_it = bl.end();	     \
+  struct_compat_it.advance(-1);				     \
   ceph_le32 struct_len;				             \
   struct_len = 0;                                            \
   ::encode(struct_len, bl);				     \
@@ -708,11 +744,18 @@ inline void decode(std::deque<T>& ls, bufferlist::iterator& p)
  * finish encoding block
  *
  * @param bl bufferlist we were encoding to
+ * @param new_struct_compat struct-compat value to use
  */
-#define ENCODE_FINISH(bl)						\
+#define ENCODE_FINISH_NEW_COMPAT(bl, new_struct_compat)			\
   } while (false);							\
   struct_len = bl.length() - struct_len_it.get_off() - sizeof(struct_len); \
-  struct_len_it.copy_in(4, (char *)&struct_len);
+  struct_len_it.copy_in(4, (char *)&struct_len);			\
+  if (new_struct_compat) {						\
+    struct_compat = new_struct_compat;					\
+    struct_compat_it.copy_in(1, (char *)&struct_compat);		\
+  }
+
+#define ENCODE_FINISH(bl) ENCODE_FINISH_NEW_COMPAT(bl, 0)
 
 #define DECODE_ERR_VERSION(func, v)			\
   "" #func " unknown encoding version > " #v
