@@ -6,9 +6,9 @@
 #include <list>
 #include <map>
 #include <set>
-#include <tr1/memory>
 #include <vector>
 #include <utility>
+#include "memory.h"
 #include "buffer.h"
 
 #include "librados.h"
@@ -80,7 +80,7 @@ namespace librados
 
   private:
     void get_next();
-    std::tr1::shared_ptr < ObjListCtx > ctx;
+    ceph::shared_ptr < ObjListCtx > ctx;
     std::pair<std::string, std::string> cur_obj;
   };
 
