@@ -72,8 +72,12 @@ librados::RadosClient::RadosClient(CephContext *cct_)
     state(DISCONNECTED),
     monclient(cct_),
     messenger(NULL),
+    instance_id(0),
     objecter(NULL),
+    osdmap_epoch(0),
+    pool_cache_epoch(0),
     lock("librados::RadosClient::lock"),
+    pool_cache_rwl("librados::RadosClient::pool_cache_rwl"),
     timer(cct, lock),
     refcnt(1),
     log_last_version(0), log_cb(NULL), log_cb_arg(NULL),
@@ -84,11 +88,35 @@ librados::RadosClient::RadosClient(CephContext *cct_)
 
 int64_t librados::RadosClient::lookup_pool(const char *name)
 {
-  Mutex::Locker l(lock);
+  pool_cache_rwl.get_read();
+  if (pool_cache_epoch && pool_cache_epoch == osdmap_epoch) {
+    map<string, int64_t>::iterator iter = pool_cache.find(name);
+    if (iter != pool_cache.end()) {
+      uint64_t val = iter->second;
+      pool_cache_rwl.unlock();
+      return val;
+    }
+  }
+
+  pool_cache_rwl.unlock();
+
+  lock.Lock();
+
   wait_for_osdmap();
   int64_t ret = osdmap.lookup_pg_pool_name(name);
-  if (ret < 0)
+  pool_cache_rwl.get_write();
+  lock.Unlock();
+  if (ret < 0) {
+    pool_cache_rwl.unlock();
     return -ENOENT;
+  }
+
+  if (pool_cache_epoch != osdmap_epoch) {
+    pool_cache.clear();
+    pool_cache_epoch = osdmap_epoch;
+  }
+  pool_cache[name] = ret;
+  pool_cache_rwl.unlock();
   return ret;
 }
 
@@ -131,6 +159,26 @@ int librados::RadosClient::get_fsid(std::string *s)
   return 0;
 }
 
+int librados::RadosClient::ping_monitor(const string mon_id, string *result)
+{
+  int err = 0;
+  /* If we haven't yet connected, we have no way of telling whether we
+   * already built monc's initial monmap.  IF we are in CONNECTED state,
+   * then it is safe to assume that we went through connect(), which does
+   * build a monmap.
+   */
+  if (state != CONNECTED) {
+    ldout(cct, 10) << __func__ << " build monmap" << dendl;
+    err = monclient.build_initial_monmap();
+  }
+  if (err < 0) {
+    return err;
+  }
+
+  err = monclient.ping_monitor(mon_id, result);
+  return err;
+}
+
 int librados::RadosClient::connect()
 {
   common_init_finish(cct);
@@ -207,6 +255,7 @@ int librados::RadosClient::connect()
   finisher.start();
 
   state = CONNECTED;
+  instance_id = monclient.get_global_id();
 
   lock.Unlock();
 
@@ -235,6 +284,7 @@ void librados::RadosClient::shutdown()
     objecter->shutdown_locked();
   }
   state = DISCONNECTED;
+  instance_id = 0;
   timer.shutdown();   // will drop+retake lock
   lock.Unlock();
   monclient.shutdown();
@@ -249,11 +299,7 @@ void librados::RadosClient::shutdown()
 
 uint64_t librados::RadosClient::get_instance_id()
 {
-  Mutex::Locker l(lock);
-  if (state == DISCONNECTED)
-    return 0;
-  uint64_t id = monclient.get_global_id();
-  return id;
+  return instance_id;
 }
 
 librados::RadosClient::~RadosClient()
@@ -321,6 +367,9 @@ bool librados::RadosClient::_dispatch(Message *m)
     break;
   case CEPH_MSG_OSD_MAP:
     objecter->handle_osd_map(static_cast<MOSDMap*>(m));
+    pool_cache_rwl.get_write();
+    osdmap_epoch = osdmap.get_epoch();
+    pool_cache_rwl.unlock();
     cond.Signal();
     break;
   case MSG_GETPOOLSTATSREPLY:
@@ -368,6 +417,24 @@ void librados::RadosClient::wait_for_osdmap()
   }
 }
 
+int librados::RadosClient::wait_for_latest_osdmap()
+{
+  Mutex mylock("RadosClient::wait_for_latest_osdmap");
+  Cond cond;
+  bool done;
+
+  lock.Lock();
+  objecter->wait_for_latest_osdmap(new C_SafeCond(&mylock, &cond, &done));
+  lock.Unlock();
+
+  mylock.Lock();
+  while (!done)
+    cond.Wait(mylock);
+  mylock.Unlock();
+
+  return 0;
+}
+
 int librados::RadosClient::pool_list(std::list<std::string>& v)
 {
   Mutex::Locker l(lock);
@@ -459,7 +526,6 @@ int librados::RadosClient::pool_create_async(string& name, PoolAsyncCompletionIm
   Context *onfinish = new C_PoolAsync_Safe(c);
   int r = objecter->create_pool(name, onfinish, auid, crush_rule);
   if (r < 0) {
-    delete c;
     delete onfinish;
   }
   return r;
@@ -505,7 +571,6 @@ int librados::RadosClient::pool_delete_async(const char *name, PoolAsyncCompleti
   Context *onfinish = new C_PoolAsync_Safe(c);
   int r = objecter->delete_pool(tmp_pool_id, onfinish);
   if (r < 0) {
-    delete c;
     delete onfinish;
   }
   return r;
@@ -574,8 +639,8 @@ void librados::RadosClient::watch_notify(MWatchNotify *m)
 }
 
 int librados::RadosClient::mon_command(const vector<string>& cmd,
-					      bufferlist &inbl,
-					      bufferlist *outbl, string *outs)
+				       const bufferlist &inbl,
+				       bufferlist *outbl, string *outs)
 {
   Mutex mylock("RadosClient::mon_command::mylock");
   Cond cond;
@@ -593,8 +658,8 @@ int librados::RadosClient::mon_command(const vector<string>& cmd,
 }
 
 int librados::RadosClient::mon_command(int rank, const vector<string>& cmd,
-					      bufferlist &inbl,
-					      bufferlist *outbl, string *outs)
+				       const bufferlist &inbl,
+				       bufferlist *outbl, string *outs)
 {
   Mutex mylock("RadosClient::mon_command::mylock");
   Cond cond;
@@ -612,8 +677,8 @@ int librados::RadosClient::mon_command(int rank, const vector<string>& cmd,
 }
 
 int librados::RadosClient::mon_command(string name, const vector<string>& cmd,
-					      bufferlist &inbl,
-					      bufferlist *outbl, string *outs)
+				       const bufferlist &inbl,
+				       bufferlist *outbl, string *outs)
 {
   Mutex mylock("RadosClient::mon_command::mylock");
   Cond cond;
@@ -631,8 +696,8 @@ int librados::RadosClient::mon_command(string name, const vector<string>& cmd,
 }
 
 int librados::RadosClient::osd_command(int osd, vector<string>& cmd,
-					bufferlist& inbl,
-					bufferlist *poutbl, string *prs)
+				       const bufferlist& inbl,
+				       bufferlist *poutbl, string *prs)
 {
   Mutex mylock("RadosClient::osd_command::mylock");
   Cond cond;
@@ -658,8 +723,8 @@ int librados::RadosClient::osd_command(int osd, vector<string>& cmd,
 }
 
 int librados::RadosClient::pg_command(pg_t pgid, vector<string>& cmd,
-					bufferlist& inbl,
-					bufferlist *poutbl, string *prs)
+				      const bufferlist& inbl,
+				      bufferlist *poutbl, string *prs)
 {
   Mutex mylock("RadosClient::pg_command::mylock");
   Cond cond;
