@@ -47,10 +47,10 @@ class Message;
 class CInode;
 class MDCache;
 class LogSegment;
-class SnapRealm;
+struct SnapRealm;
 class Session;
 class MClientCaps;
-class ObjectOperation;
+struct ObjectOperation;
 class EMetaBlob;
 
 ostream& operator<<(ostream& out, CInode& in);
@@ -156,6 +156,8 @@ public:
   static const int STATE_STRAYPINNED = (1<<16);
   static const int STATE_FROZENAUTHPIN = (1<<17);
   static const int STATE_DIRTYPOOL =   (1<<18);
+  // orphan inode needs notification of releasing reference
+  static const int STATE_ORPHAN =	STATE_NOTIFYREF;
 
   static const int MASK_STATE_EXPORTED =
     (STATE_DIRTY|STATE_NEEDSRECOVER|STATE_DIRTYPARENT|STATE_DIRTYPOOL);
@@ -173,7 +175,7 @@ public:
   static const uint64_t WAIT_ANY_MASK	= (uint64_t)(-1);
 
   // misc
-  static const int EXPORT_NONCE = 1; // nonce given to replicas created by export
+  static const unsigned EXPORT_NONCE = 1; // nonce given to replicas created by export
 
   ostream& print_db_line_prefix(ostream& out);
 
@@ -812,6 +814,7 @@ public:
   }
   void first_get();
   void last_put();
+  void _put();
 
 
   // -- hierarchy stuff --
