@@ -39,6 +39,8 @@
 #include "common/cmdparse.h"
 #include "include/str_list.h"
 
+#include "mds/mdstypes.h"
+
 #define dout_subsys ceph_subsys_mon
 #undef dout_prefix
 #define dout_prefix _prefix(_dout, mon, mdsmap)
@@ -66,7 +68,7 @@ void MDSMonitor::create_new_fs(MDSMap &m, int metadata_pool, int data_pool)
   m.data_pools.insert(data_pool);
   m.metadata_pool = metadata_pool;
   m.cas_pool = -1;
-  m.compat = get_mdsmap_compat_set();
+  m.compat = get_mdsmap_compat_set_default();
 
   m.session_timeout = g_conf->mds_session_timeout;
   m.session_autoclose = g_conf->mds_session_autoclose;
@@ -80,7 +82,7 @@ void MDSMonitor::create_new_fs(MDSMap &m, int metadata_pool, int data_pool)
 void MDSMonitor::create_initial()
 {
   dout(10) << "create_initial" << dendl;
-  create_new_fs(pending_mdsmap, CEPH_METADATA_RULE, CEPH_DATA_RULE);
+  create_new_fs(pending_mdsmap, MDS_METADATA_POOL, MDS_DATA_POOL);
 }
 
 
@@ -817,13 +819,85 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
     }
 
   } else if (prefix == "mds set_max_mds") {
+    // NOTE: see also "mds set max_mds", which can modify the same field.
     int64_t maxmds;
     if (!cmd_getval(g_ceph_context, cmdmap, "maxmds", maxmds) || maxmds < 0)
       goto out;
     pending_mdsmap.max_mds = maxmds;
     r = 0;
     ss << "max_mds = " << pending_mdsmap.max_mds;
-
+  } else if (prefix == "mds set") {
+    string var;
+    if (!cmd_getval(g_ceph_context, cmdmap, "var", var) < 0)
+      goto out;
+    string val;
+    string interr;
+    int64_t n = 0;
+    if (!cmd_getval(g_ceph_context, cmdmap, "val", val))
+      goto out;
+    // we got a string.  see if it contains an int.
+    n = strict_strtoll(val.c_str(), 10, &interr);
+    if (var == "max_mds") {
+      // NOTE: see also "mds set_max_mds", which can modify the same field.
+      if (interr.length())
+	goto out;
+      pending_mdsmap.max_mds = n;
+    } else if (var == "inline_data") {
+      if (val == "true" || val == "yes" || (!interr.length() && n == 1)) {
+	string confirm;
+	if (!cmd_getval(g_ceph_context, cmdmap, "confirm", confirm) ||
+	    confirm != "--yes-i-really-mean-it") {
+	  ss << "inline data is new and experimental; you must specify --yes-i-really-mean-it";
+	  r = -EPERM;
+	  goto out;
+	}
+	ss << "inline data enabled";
+	pending_mdsmap.set_inline_data_enabled(true);
+	pending_mdsmap.compat.incompat.insert(MDS_FEATURE_INCOMPAT_INLINE);
+      } else if (val == "false" || val == "no" ||
+		 (!interr.length() && n == 0)) {
+	ss << "inline data disabled";
+	pending_mdsmap.set_inline_data_enabled(false);
+      } else {
+	ss << "value must be false|no|0 or true|yes|1";
+	r = -EINVAL;
+	goto out;
+      }
+    } else if (var == "max_file_size") {
+      if (interr.length()) {
+	ss << var << " requires an integer value";
+	goto out;
+      }
+      if (n < CEPH_MIN_STRIPE_UNIT) {
+	r = -ERANGE;
+	ss << var << " must at least " << CEPH_MIN_STRIPE_UNIT;
+	goto out;
+      }
+      pending_mdsmap.max_file_size = n;
+    } else if (var == "allow_new_snaps") {
+      if (val == "false" || val == "no" || (interr.length() == 0 && n == 0)) {
+	pending_mdsmap.clear_snaps_allowed();
+	ss << "disabled new snapshots";
+      } else if (val == "true" || val == "yes" || (interr.length() == 0 && n == 1)) {
+	string confirm;
+	if (!cmd_getval(g_ceph_context, cmdmap, "confirm", confirm) ||
+	    confirm != "--yes-i-really-mean-it") {
+	  ss << "Snapshots are unstable and will probably break your FS! Set to --yes-i-really-mean-it if you are sure you want to enable them";
+	  r = -EPERM;
+	  goto out;
+	}
+	pending_mdsmap.set_snaps_allowed();
+	ss << "enabled new snapshots";
+      } else {
+	ss << "value must be true|yes|1 or false|no|0";
+	r = -EINVAL;
+	goto out;
+      }
+    } else {
+      ss << "unknown variable " << var;
+      goto out;
+    }
+    r = 0;
   } else if (prefix == "mds setmap") {
     MDSMap map;
     map.decode(m->get_data());
@@ -836,7 +910,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       map.epoch = pending_mdsmap.epoch;  // make sure epoch is correct
       pending_mdsmap = map;
       string rs = "set mds map";
-      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_last_committed()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs,
+						get_last_committed() + 1));
       return true;
     } else {
       ss << "next mdsmap epoch " << pending_mdsmap.epoch << " != " << e;
@@ -855,7 +930,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       ss << "set mds gid " << gid << " to state " << state << " " << ceph_mds_state_name(state);
       string rs;
       getline(ss, rs);
-      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_last_committed()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs,
+						get_last_committed() + 1));
       return true;
     }
 
@@ -885,7 +961,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       ss << "removed mds gid " << gid;
       string rs;
       getline(ss, rs);
-      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_last_committed()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs,
+						get_last_committed() + 1));
       return true;
     }
   } else if (prefix == "mds rmfailed") {
@@ -896,7 +973,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
     ss << "removed failed mds." << w;
     string rs;
     getline(ss, rs);
-    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_last_committed()));
+    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs,
+					      get_last_committed() + 1));
     return true;
   } else if (prefix == "mds cluster_down") {
     if (pending_mdsmap.test_flag(CEPH_MDSMAP_DOWN)) {
@@ -937,36 +1015,6 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       r = 0;
     }
 
-  } else if (prefix == "mds set") {
-    string key;
-    cmd_getval(g_ceph_context, cmdmap, "key", key);
-    string sure;
-    cmd_getval(g_ceph_context, cmdmap, "sure", sure);
-    if (key == "allow_new_snaps") {
-      if (sure != "--yes-i-really-mean-it") {
-	ss << "Snapshots are unstable and will probably break your FS! Add --yes-i-really-mean-it if you are sure";
-	r = -EPERM;
-      } else {
-	pending_mdsmap.set_snaps_allowed();
-	ss << "turned on snaps";
-	r = 0;
-      }
-    }
-  } else if (prefix == "mds unset") {
-    string key;
-    cmd_getval(g_ceph_context, cmdmap, "key", key);
-    string sure;
-    cmd_getval(g_ceph_context, cmdmap, "sure", sure);
-    if (key == "allow_new_snaps") {
-      if (sure != "--yes-i-really-mean-it") {
-	ss << "this won't get rid of snapshots or restore the cluster if it's broken. Add --yes-i-really-mean-it if you are sure";
-	r = -EPERM;
-      } else {
-	pending_mdsmap.clear_snaps_allowed();
-	ss << "disabled new snapshots";
-	r = 0;
-      }
-    }
   } else if (prefix == "mds add_data_pool") {
     string poolname;
     cmd_getval(g_ceph_context, cmdmap, "pool", poolname);
@@ -1024,7 +1072,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
       ss << "new fs with metadata pool " << metadata << " and data pool " << data;
       string rs;
       getline(ss, rs);
-      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_last_committed()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs,
+						get_last_committed() + 1));
       return true;
     }
   } else {
@@ -1036,7 +1085,8 @@ bool MDSMonitor::prepare_command(MMonCommand *m)
 
   if (r >= 0) {
     // success.. delay reply
-    wait_for_finished_proposal(new Monitor::C_Command(mon, m, r, rs, get_last_committed()));
+    wait_for_finished_proposal(new Monitor::C_Command(mon, m, r, rs,
+					      get_last_committed() + 1));
     return true;
   } else {
     // reply immediately
@@ -1216,14 +1266,15 @@ void MDSMonitor::tick()
 		   << " " << ceph_mds_state_name(info.state)
 		   << " (laggy)" << dendl;
 	  pending_mdsmap.mds_info.erase(gid);
+	  do_propose = true;
 	} else if (!info.laggy()) {
 	  dout(10) << " marking " << gid << " " << info.addr << " mds." << info.rank << "." << info.inc
 		   << " " << ceph_mds_state_name(info.state)
 		   << " laggy" << dendl;
 	  info.laggy_since = now;
+	  do_propose = true;
 	}
 	last_beacon.erase(gid);
-	do_propose = true;
       }
     }
 
