@@ -28,6 +28,7 @@ using namespace std;
 #include "PaxosService.h"
 #include "include/types.h"
 #include "include/utime.h"
+#include "include/histogram.h"
 #include "msg/Messenger.h"
 #include "common/config.h"
 #include "mon/MonitorDBStore.h"
@@ -47,20 +48,32 @@ public:
 
   bool need_check_down_pgs;
 
+  epoch_t last_map_pg_create_osd_epoch;
+
+
 private:
   PGMap::Incremental pending_inc;
 
+  const char *pgmap_meta_prefix;
+  const char *pgmap_pg_prefix;
+  const char *pgmap_osd_prefix;
+
   void create_initial();
   void update_from_paxos(bool *need_bootstrap);
-  void init();
+  void upgrade_format();
+  void on_upgrade();
+  void post_paxos_update();
   void handle_osd_timeouts();
   void create_pending();  // prepare a new pending
   // propose pending update to peers
-  void update_trim();
-  void encode_pending(MonitorDBStore::Transaction *t);
-  virtual void encode_full(MonitorDBStore::Transaction *t);
+  version_t get_trim_to();
   void update_logger();
 
+  void encode_pending(MonitorDBStore::Transaction *t);
+  void read_pgmap_meta();
+  void read_pgmap_full();
+  void apply_pgmap_delta(bufferlist& bl);
+
   bool preprocess_query(PaxosServiceMessage *m);  // true if processed.
   bool prepare_update(PaxosServiceMessage *m);
 
@@ -137,9 +150,22 @@ private:
 
 public:
   PGMonitor(Monitor *mn, Paxos *p, const string& service_name)
-  : PaxosService(mn, p, service_name), need_check_down_pgs(false) { }
+    : PaxosService(mn, p, service_name),
+      need_check_down_pgs(false),
+      last_map_pg_create_osd_epoch(0),
+      pgmap_meta_prefix("pgmap_meta"),
+      pgmap_pg_prefix("pgmap_pg"),
+      pgmap_osd_prefix("pgmap_osd")
+  { }
   ~PGMonitor() { }
 
+  virtual void get_store_prefixes(set<string>& s) {
+    s.insert(get_service_name());
+    s.insert(pgmap_meta_prefix);
+    s.insert(pgmap_pg_prefix);
+    s.insert(pgmap_osd_prefix);
+  }
+
   virtual void on_restart();
 
   /* Courtesy function provided by PaxosService, called when an election
@@ -147,6 +173,14 @@ public:
    * haven't lost any PGs from new pools. */
   virtual void on_active();
 
+  bool should_stash_full() {
+    return false;  // never
+  }
+  virtual void encode_full(MonitorDBStore::Transaction *t) {
+    assert(0 == "unimplemented encode_full");
+  }
+
+
   void tick();  // check state, take actions
 
   void check_osd_map(epoch_t epoch);
@@ -156,6 +190,10 @@ public:
 
   void dump_info(Formatter *f);
 
+  int _warn_slow_request_histogram(const pow2_hist_t& h, string suffix,
+				   list<pair<health_status_t,string> >& summary,
+				   list<pair<health_status_t,string> > *detail) const;
+
   void get_health(list<pair<health_status_t,string> >& summary,
 		  list<pair<health_status_t,string> > *detail) const;
   void check_full_osd_health(list<pair<health_status_t,string> >& summary,
