@@ -11,9 +11,20 @@
  * Foundation.  See file COPYING.
  *
  */
+#include "acconfig.h"
+
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 
 #include "include/types.h"
 #include "include/stringify.h"
+#include "include/unordered_map.h"
+#include "include/memory.h"
 #include "common/errno.h"
 #include "MemStore.h"
 
@@ -56,7 +67,7 @@ int MemStore::_save()
   Mutex::Locker l(apply_lock); // block any writer
   dump_all();
   set<coll_t> collections;
-  for (hash_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
+  for (ceph::unordered_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
        p != coll_map.end();
        ++p) {
     dout(20) << __func__ << " coll " << p->first << " " << p->second << dendl;
@@ -95,7 +106,7 @@ void MemStore::dump_all()
 void MemStore::dump(Formatter *f)
 {
   f->open_array_section("collections");
-  for (hash_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
+  for (ceph::unordered_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
        p != coll_map.end();
        ++p) {
     f->open_object_section("collection");
@@ -218,16 +229,16 @@ int MemStore::statfs(struct statfs *st)
   return 0;
 }
 
-filestore_perf_stat_t MemStore::get_cur_stats()
+objectstore_perf_stat_t MemStore::get_cur_stats()
 {
   // fixme
-  return filestore_perf_stat_t();
+  return objectstore_perf_stat_t();
 }
 
 MemStore::CollectionRef MemStore::get_collection(coll_t cid)
 {
   RWLock::RLocker l(coll_lock);
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp == coll_map.end())
     return CollectionRef();
   return cp->second;
@@ -245,7 +256,9 @@ bool MemStore::exists(coll_t cid, const ghobject_t& oid)
     return false;
   RWLock::RLocker l(c->lock);
 
-  return (c->get_object(oid) != NULL);
+  // Perform equivalent of c->get_object_(oid) != NULL. In C++11 the
+  // shared_ptr needs to be compared to nullptr.
+  return (bool)c->get_object(oid);
 }
 
 int MemStore::stat(
@@ -374,7 +387,7 @@ int MemStore::list_collections(vector<coll_t>& ls)
 {
   dout(10) << __func__ << dendl;
   RWLock::RLocker l(coll_lock);
-  for (hash_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
+  for (ceph::unordered_map<coll_t,CollectionRef>::iterator p = coll_map.begin();
        p != coll_map.end();
        ++p) {
     ls.push_back(p->first);
@@ -648,7 +661,9 @@ int MemStore::queue_transactions(Sequencer *osr,
 
   for (list<Transaction*>::iterator p = tls.begin(); p != tls.end(); ++p) {
     // poke the TPHandle heartbeat just to exercise that code path
-    handle->reset_tp_timeout();
+    if (handle)
+      handle->reset_tp_timeout();
+
     _do_transaction(**p);
   }
 
@@ -997,8 +1012,9 @@ int MemStore::_touch(coll_t cid, const ghobject_t& oid)
 
   ObjectRef o = c->get_object(oid);
   if (!o) {
-    c->object_map[oid].reset(new Object);
-    c->object_hash[oid].reset(new Object);
+    o.reset(new Object);
+    c->object_map[oid] = o;
+    c->object_hash[oid] = o;
   }
   return 0;
 }
@@ -1304,7 +1320,7 @@ int MemStore::_create_collection(coll_t cid)
 {
   dout(10) << __func__ << " " << cid << dendl;
   RWLock::WLocker l(coll_lock);
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp != coll_map.end())
     return -EEXIST;
   coll_map[cid].reset(new Collection);
@@ -1315,7 +1331,7 @@ int MemStore::_destroy_collection(coll_t cid)
 {
   dout(10) << __func__ << " " << cid << dendl;
   RWLock::WLocker l(coll_lock);
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp == coll_map.end())
     return -ENOENT;
   {
@@ -1379,7 +1395,7 @@ int MemStore::_collection_setattr(coll_t cid, const char *name,
 				  const void *value, size_t size)
 {
   dout(10) << __func__ << " " << cid << " " << name << dendl;
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp == coll_map.end())
     return -ENOENT;
   RWLock::WLocker l(cp->second->lock);
@@ -1391,7 +1407,7 @@ int MemStore::_collection_setattr(coll_t cid, const char *name,
 int MemStore::_collection_setattrs(coll_t cid, map<string,bufferptr> &aset)
 {
   dout(10) << __func__ << " " << cid << dendl;
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp == coll_map.end())
     return -ENOENT;
   RWLock::WLocker l(cp->second->lock);
@@ -1407,7 +1423,7 @@ int MemStore::_collection_setattrs(coll_t cid, map<string,bufferptr> &aset)
 int MemStore::_collection_rmattr(coll_t cid, const char *name)
 {
   dout(10) << __func__ << " " << cid << " " << name << dendl;
-  hash_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
+  ceph::unordered_map<coll_t,CollectionRef>::iterator cp = coll_map.find(cid);
   if (cp == coll_map.end())
     return -ENOENT;
   RWLock::WLocker l(cp->second->lock);
