@@ -47,33 +47,6 @@ namespace ceph {
 
 class Logger;
 
-enum {
-  l_os_first = 84000,
-  l_os_jq_max_ops,
-  l_os_jq_ops,
-  l_os_j_ops,
-  l_os_jq_max_bytes,
-  l_os_jq_bytes,
-  l_os_j_bytes,
-  l_os_j_lat,
-  l_os_j_wr,
-  l_os_j_wr_bytes,
-  l_os_oq_max_ops,
-  l_os_oq_ops,
-  l_os_ops,
-  l_os_oq_max_bytes,
-  l_os_oq_bytes,
-  l_os_bytes,
-  l_os_apply_lat,
-  l_os_committing,
-  l_os_commit,
-  l_os_commit_len,
-  l_os_commit_lat,
-  l_os_j_full,
-  l_os_queue_lat,
-  l_os_last,
-};
-
 
 static inline void encode(const map<string,bufferptr> *attrset, bufferlist &bl) {
   ::encode(*attrset, bl);
@@ -98,7 +71,7 @@ public:
 
   Logger *logger;
 
-  virtual filestore_perf_stat_t get_cur_stats() = 0;
+  virtual objectstore_perf_stat_t get_cur_stats() = 0;
 
   /**
    * a sequencer orders transactions
@@ -369,7 +342,7 @@ public:
 	} else {
 	  ::decode(oid, p);
 	  if (use_pool_override && pool_override != -1 &&
-	      oid.hobj.pool == -1) {
+	      !oid.hobj.is_max() && oid.hobj.pool == -1) {
 	    oid.hobj.pool = pool_override;
 	  }
 	}
@@ -582,6 +555,7 @@ public:
       ::encode(oldoid, tbl);
       ::encode(cid, tbl);
       ::encode(oid, tbl);
+      ops++;
     }
 
     void collection_setattr(coll_t cid, const char* name, bufferlist& val) {
@@ -810,8 +784,8 @@ public:
   }
   unsigned apply_transactions(Sequencer *osr, list<Transaction*>& tls, Context *ondisk=0);
 
-  int queue_transaction(Sequencer *osr, Transaction* t,
-                        ThreadPool::TPHandle *handle = NULL) {
+  int queue_transaction_and_cleanup(Sequencer *osr, Transaction* t,
+				    ThreadPool::TPHandle *handle = NULL) {
     list<Transaction *> tls;
     tls.push_back(t);
     return queue_transactions(osr, tls, new C_DeleteTransaction(t),
