@@ -21,7 +21,7 @@
 #include "common/Mutex.h"
 #include "include/xlist.h"
 #include "msg/Message.h"
-#include <tr1/memory>
+#include "include/memory.h"
 #include "common/TrackedOp.h"
 
 /**
@@ -78,6 +78,7 @@ private:
   osd_reqid_t reqid;
   uint8_t hit_flag_points;
   uint8_t latest_flag_point;
+  utime_t dequeued_time;
   static const uint8_t flag_queued_for_pg=1 << 0;
   static const uint8_t flag_reached_pg =  1 << 1;
   static const uint8_t flag_delayed =     1 << 2;
@@ -151,13 +152,20 @@ public:
     latest_flag_point = flag_commit_sent;
   }
 
+  utime_t get_dequeued_time() const {
+    return dequeued_time;
+  }
+  void set_dequeued_time(utime_t deq_time) {
+    dequeued_time = deq_time;
+  }
+
   osd_reqid_t get_reqid() const {
     return reqid;
   }
 
   void init_from_message();
 
-  typedef std::tr1::shared_ptr<OpRequest> Ref;
+  typedef ceph::shared_ptr<OpRequest> Ref;
 };
 
 typedef OpRequest::Ref OpRequestRef;
