@@ -40,7 +40,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <time.h>
-#include <tr1/memory>
+#include "include/memory.h"
 #include <sys/ioctl.h>
 
 #include "include/rbd_types.h"
@@ -57,6 +57,8 @@
 #include <sys/param.h>
 #endif
 
+#include <blkid/blkid.h>
+
 #define MAX_SECRET_LEN 1000
 #define MAX_POOL_NAME_SIZE 128
 
@@ -68,7 +70,8 @@ static string dir_info_oid = RBD_INFO;
 bool udevadm_settle = true;
 bool progress = true;
 bool resize_allow_shrink = false;
-bool read_only = false;
+
+map<string, string> map_options; // -o / --options map
 
 #define dout_subsys ceph_subsys_rbd
 
@@ -152,6 +155,7 @@ void usage()
 "  --pretty-format                    make json or xml output more readable\n"
 "  --no-settle                        do not wait for udevadm to settle on map/unmap\n"
 "  --no-progress                      do not show progress for long-running commands\n"
+"  -o, --options <map-options>        options to use when mapping an image\n"
 "  --read-only                        set device readonly when mapping image\n"
 "  --allow-shrink                     allow shrinking of an image when resizing\n";
 }
@@ -247,6 +251,8 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag,
 {
   std::vector<string> names;
   int r = rbd.list(io_ctx, names);
+  if (r == -ENOENT)
+    r = 0;
   if (r < 0)
     return r;
 
@@ -867,6 +873,11 @@ void rbd_bencher_completion(void *vc, void *pc)
   librbd::RBD::AioCompletion *c = (librbd::RBD::AioCompletion *)vc;
   rbd_bencher *b = static_cast<rbd_bencher *>(pc);
   //cout << "complete " << c << std::endl;
+  int ret = c->get_return_value();
+  if (ret != 0) {
+    cout << "write error: " << cpp_strerror(ret) << std::endl;
+    assert(0 == ret);
+  }
   b->lock.Lock();
   b->in_flight--;
   b->cond.Signal();
@@ -904,19 +915,34 @@ static int do_bench_write(librbd::Image& image, uint64_t io_size,
   uint64_t size = 0;
   image.size(&size);
 
+  vector<uint64_t> thread_offset;
+  uint64_t i;
+  uint64_t start_pos;
+
+  // disturb all thread's offset, used by seq write
+  for (i = 0; i < io_threads; i++) {
+    start_pos = (rand() % (size / io_size)) * io_size;
+    thread_offset.push_back(start_pos);
+  }
+
   printf("  SEC       OPS   OPS/SEC   BYTES/SEC\n");
   uint64_t off;
   for (off = 0; off < io_bytes; off += io_size) {
     b.wait_for(io_threads - 1);
-    uint64_t i = 0;
-    uint64_t real_off = off;
-    if (pattern == "rand") {
-      real_off = (rand() % (size / io_size)) * io_size;
-    }
-    while (i < io_threads &&
-	   b.start_write(io_threads, real_off, io_size, bl)) {
+    i = 0;
+    while (i < io_threads && off < io_bytes &&
+	   b.start_write(io_threads, thread_offset[i], io_size, bl)) {
       ++i;
       ++ios;
+      off += io_size;
+
+      if (pattern == "rand") {
+        thread_offset[i] = (rand() % (size / io_size)) * io_size;
+      } else {
+        thread_offset[i] += io_size;
+        if (thread_offset[i] + io_size > size)
+          thread_offset[i] = 0;
+      }
     }
 
     utime_t now = ceph_clock_now(NULL);
@@ -1642,13 +1668,8 @@ static int do_kernel_add(const char *poolname, const char *imgname,
       oss << ",";
   }
 
-  if (read_only)
-    oss << " ro,";
-  else
-    oss << " ";
-
   const char *user = g_conf->name.get_id().c_str();
-  oss << "name=" << user;
+  oss << " name=" << user;
 
   char key_name[strlen(user) + strlen("client.") + 1];
   snprintf(key_name, sizeof(key_name), "client.%s", user);
@@ -1684,6 +1705,18 @@ static int do_kernel_add(const char *poolname, const char *imgname,
     oss << ",key=" << key_name;
   }
 
+  for (map<string, string>::const_iterator it = map_options.begin();
+       it != map_options.end();
+       ++it) {
+    // for compatibility with < 3.7 kernels, assume that rw is on by
+    // default and omit it even if it was specified by the user
+    // (see ceph.git commit fb0f1986449b)
+    if (it->first == "rw" && it->second == "rw")
+      continue;
+
+    oss << "," << it->second;
+  }
+
   oss << " " << poolname << " " << imgname;
 
   if (snapname) {
@@ -1693,7 +1726,17 @@ static int do_kernel_add(const char *poolname, const char *imgname,
   // modprobe the rbd module if /sys/bus/rbd doesn't exist
   struct stat sb;
   if ((stat("/sys/bus/rbd", &sb) < 0) || (!S_ISDIR(sb.st_mode))) {
-    r = system("/sbin/modprobe rbd");
+    // turn on single-major device number allocation scheme if the
+    // kernel supports it
+    const char *cmd = "/sbin/modprobe rbd";
+    r = system("/sbin/modinfo -F parm rbd | /bin/grep -q ^single_major:");
+    if (r == 0) {
+      cmd = "/sbin/modprobe rbd single_major=Y";
+    } else if (r < 0) {
+      cerr << "rbd: error executing modinfo as shell command!" << std::endl;
+    }
+
+    r = system(cmd);
     if (r) {
       if (r < 0)
         cerr << "rbd: error executing modprobe as shell command!" << std::endl;
@@ -1703,16 +1746,29 @@ static int do_kernel_add(const char *poolname, const char *imgname,
     }
   }
 
-  // write to /sys/bus/rbd/add
-  int fd = open("/sys/bus/rbd/add", O_WRONLY);
+  // 'add' interface is deprecated, use 'add_single_major' if it's
+  // available
+  //
+  // ('add' and 'add_single_major' interfaces are identical, except
+  // that if rbd kernel module is new enough and is configured to use
+  // single-major scheme, 'add' is disabled in order to prevent old
+  // userspace from doing weird things at unmap time)
+  int fd = open("/sys/bus/rbd/add_single_major", O_WRONLY);
   if (fd < 0) {
-    r = -errno;
-    if (r == -ENOENT) {
-      cerr << "rbd: /sys/bus/rbd/add does not exist!" << std::endl
-	   << "Did you run 'modprobe rbd' or is your rbd module too old?"
-	   << std::endl;
+    if (errno == ENOENT) {
+      fd = open("/sys/bus/rbd/add", O_WRONLY);
+      if (fd < 0) {
+        r = -errno;
+        if (r == -ENOENT) {
+          cerr << "rbd: /sys/bus/rbd/add does not exist!" << std::endl
+               << "Did you run 'modprobe rbd' or is your rbd module too old?"
+               << std::endl;
+        }
+        return r;
+      }
+    } else {
+      return -errno;
     }
-    return r;
   }
 
   string add = oss.str();
@@ -1771,7 +1827,7 @@ static int do_kernel_showmapped(Formatter *f)
   TextTable tbl;
 
   const char *devices_path = "/sys/bus/rbd/devices";
-  std::tr1::shared_ptr<DIR> device_dir(opendir(devices_path), do_closedir);
+  ceph::shared_ptr<DIR> device_dir(opendir(devices_path), do_closedir);
   if (!device_dir.get()) {
     r = -errno;
     cerr << "rbd: could not open " << devices_path << ": "
@@ -1859,9 +1915,19 @@ static int do_kernel_showmapped(Formatter *f)
   return 0;
 }
 
-static int get_rbd_seq(int major_num, string &seq)
+static int get_rbd_seq(dev_t devno, string &seq)
 {
-  int r;
+  // convert devno, which might be a partition major:minor pair, into
+  // a whole disk major:minor pair
+  dev_t wholediskno;
+  int r = blkid_devno_to_wholedisk(devno, NULL, 0, &wholediskno);
+  if (r) {
+    cerr << "rbd: could not compute wholediskno: " << r << std::endl;
+    // ignore the error: devno == wholediskno most of the time, and if
+    // it turns out it's not we will fail with -ENOENT later anyway
+    wholediskno = devno;
+  }
+
   const char *devices_path = "/sys/bus/rbd/devices";
   DIR *device_dir = opendir(devices_path);
   if (!device_dir) {
@@ -1881,28 +1947,66 @@ static int get_rbd_seq(int major_num, string &seq)
     return r;
   }
 
-  char major[32];
+  int match_minor = -1;
   do {
+    char fn[strlen(devices_path) + strlen(dent->d_name) + strlen("//major") + 1];
+    char buf[32];
+
     if (strcmp(dent->d_name, ".") == 0 || strcmp(dent->d_name, "..") == 0)
       continue;
 
-    char fn[strlen(devices_path) + strlen(dent->d_name) + strlen("//major") + 1];
-
     snprintf(fn, sizeof(fn), "%s/%s/major", devices_path, dent->d_name);
-    r = read_file(fn, major, sizeof(major));
+    r = read_file(fn, buf, sizeof(buf));
     if (r < 0) {
       cerr << "rbd: could not read major number from " << fn << ": "
 	   << cpp_strerror(-r) << std::endl;
       continue;
     }
+    string err;
+    int cur_major = strict_strtol(buf, 10, &err);
+    if (!err.empty()) {
+      cerr << err << std::endl;
+      cerr << "rbd: could not parse major number read from " << fn << ": "
+           << cpp_strerror(-r) << std::endl;
+      continue;
+    }
+    if (cur_major != (int)major(wholediskno))
+      continue;
 
-    int cur_major = atoi(major);
-    if (cur_major == major_num) {
-      seq = string(dent->d_name);
-      closedir(device_dir);
-      return 0;
+    if (match_minor == -1) {
+      // matching minors in addition to majors is not necessary unless
+      // single-major scheme is turned on, but, if the kernel supports
+      // it, do it anyway (blkid stuff above ensures that we always have
+      // the correct minor to match with)
+      struct stat sbuf;
+      snprintf(fn, sizeof(fn), "%s/%s/minor", devices_path, dent->d_name);
+      match_minor = (stat(fn, &sbuf) == 0);
     }
 
+    if (match_minor == 1) {
+      snprintf(fn, sizeof(fn), "%s/%s/minor", devices_path, dent->d_name);
+      r = read_file(fn, buf, sizeof(buf));
+      if (r < 0) {
+        cerr << "rbd: could not read minor number from " << fn << ": "
+             << cpp_strerror(-r) << std::endl;
+        continue;
+      }
+      int cur_minor = strict_strtol(buf, 10, &err);
+      if (!err.empty()) {
+        cerr << err << std::endl;
+        cerr << "rbd: could not parse minor number read from " << fn << ": "
+             << cpp_strerror(-r) << std::endl;
+        continue;
+      }
+      if (cur_minor != (int)minor(wholediskno))
+        continue;
+    } else {
+      assert(match_minor == 0);
+    }
+
+    seq = string(dent->d_name);
+    closedir(device_dir);
+    return 0;
   } while ((dent = readdir(device_dir)));
 
   closedir(device_dir);
@@ -1911,16 +2015,14 @@ static int get_rbd_seq(int major_num, string &seq)
 
 static int do_kernel_rm(const char *dev)
 {
-  struct stat dev_stat;
-  int r = stat(dev, &dev_stat);
-  if (!S_ISBLK(dev_stat.st_mode)) {
+  struct stat sbuf;
+  if (stat(dev, &sbuf) || !S_ISBLK(sbuf.st_mode)) {
     cerr << "rbd: " << dev << " is not a block device" << std::endl;
     return -EINVAL;
   }
 
-  int major = major(dev_stat.st_rdev);
   string seq_num;
-  r = get_rbd_seq(major, seq_num);
+  int r = get_rbd_seq(sbuf.st_rdev, seq_num);
   if (r == -ENOENT) {
     cerr << "rbd: " << dev << " is not an rbd device" << std::endl;
     return -EINVAL;
@@ -1940,9 +2042,17 @@ static int do_kernel_rm(const char *dev)
     }
   }
 
-  int fd = open("/sys/bus/rbd/remove", O_WRONLY);
+  // see comment in do_kernel_add(), same goes for 'remove' vs
+  // 'remove_single_major'
+  int fd = open("/sys/bus/rbd/remove_single_major", O_WRONLY);
   if (fd < 0) {
-    return -errno;
+    if (errno == ENOENT) {
+      fd = open("/sys/bus/rbd/remove", O_WRONLY);
+      if (fd < 0)
+        return -errno;
+    } else {
+      return -errno;
+    }
   }
 
   r = safe_write(fd, seq_num.c_str(), seq_num.size());
@@ -1972,6 +2082,112 @@ static int do_kernel_rm(const char *dev)
   return r;
 }
 
+static string map_option_uuid_cb(const char *value_char)
+{
+  uuid_d u;
+  if (!u.parse(value_char))
+    return "";
+
+  ostringstream oss;
+  oss << u;
+  return oss.str();
+}
+
+static string map_option_ip_cb(const char *value_char)
+{
+  entity_addr_t a;
+  const char *endptr;
+  if (!a.parse(value_char, &endptr) ||
+      endptr != value_char + strlen(value_char)) {
+    return "";
+  }
+
+  ostringstream oss;
+  oss << a.addr;
+  return oss.str();
+}
+
+static string map_option_int_cb(const char *value_char)
+{
+  string err;
+  int d = strict_strtol(value_char, 10, &err);
+  if (!err.empty() || d < 0)
+    return "";
+
+  ostringstream oss;
+  oss << d;
+  return oss.str();
+}
+
+static void put_map_option(const string key, string val)
+{
+  map<string, string>::const_iterator it = map_options.find(key);
+  if (it != map_options.end()) {
+    cerr << "rbd: warning: redefining map option " << key << ": '"
+         << it->second << "' -> '" << val << "'" << std::endl;
+  }
+  map_options[key] = val;
+}
+
+static int put_map_option_value(const string opt, const char *value_char,
+                                string (*parse_cb)(const char *))
+{
+  if (!value_char || *value_char == '\0') {
+    cerr << "rbd: " << opt << " option requires a value" << std::endl;
+    return 1;
+  }
+
+  string value = parse_cb(value_char);
+  if (value.empty()) {
+    cerr << "rbd: invalid " << opt << " value '" << value_char << "'"
+         << std::endl;
+    return 1;
+  }
+
+  put_map_option(opt, opt + "=" + value);
+  return 0;
+}
+
+static int parse_map_options(char *options)
+{
+  for (char *this_char = strtok(options, ", ");
+       this_char != NULL;
+       this_char = strtok(NULL, ",")) {
+    char *value_char;
+
+    if ((value_char = strchr(this_char, '=')) != NULL)
+      *value_char++ = '\0';
+
+    if (!strcmp(this_char, "fsid")) {
+      if (put_map_option_value("fsid", value_char, map_option_uuid_cb))
+        return 1;
+    } else if (!strcmp(this_char, "ip")) {
+      if (put_map_option_value("ip", value_char, map_option_ip_cb))
+        return 1;
+    } else if (!strcmp(this_char, "share") || !strcmp(this_char, "noshare")) {
+      put_map_option("share", this_char);
+    } else if (!strcmp(this_char, "crc") || !strcmp(this_char, "nocrc")) {
+      put_map_option("crc", this_char);
+    } else if (!strcmp(this_char, "mount_timeout")) {
+      if (put_map_option_value("mount_timeout", value_char, map_option_int_cb))
+        return 1;
+    } else if (!strcmp(this_char, "osdkeepalive")) {
+      if (put_map_option_value("osdkeepalive", value_char, map_option_int_cb))
+        return 1;
+    } else if (!strcmp(this_char, "osd_idle_ttl")) {
+      if (put_map_option_value("osd_idle_ttl", value_char, map_option_int_cb))
+        return 1;
+    } else if (!strcmp(this_char, "rw") || !strcmp(this_char, "ro")) {
+      put_map_option("rw", this_char);
+    } else {
+      cerr << "rbd: unknown map option '" << this_char << "'" << std::endl;
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
 enum {
   OPT_NO_CMD = 0,
   OPT_LIST,
@@ -2207,8 +2423,15 @@ int main(int argc, const char **argv)
       lock_tag = strdup(val.c_str());
     } else if (ceph_argparse_flag(args, i, "--no-settle", (char *)NULL)) {
       udevadm_settle = false;
+    } else if (ceph_argparse_witharg(args, i, &val, "-o", "--options", (char*)NULL)) {
+      char *map_options = strdup(val.c_str());
+      if (parse_map_options(map_options)) {
+        cerr << "rbd: couldn't parse map options" << std::endl;
+        return EXIT_FAILURE;
+      }
     } else if (ceph_argparse_flag(args, i, "--read-only", (char *)NULL)) {
-      read_only = true;
+      // --read-only is equivalent to -o ro
+      put_map_option("rw", "ro");
     } else if (ceph_argparse_flag(args, i, "--no-progress", (char *)NULL)) {
       progress = false;
     } else if (ceph_argparse_flag(args, i , "--allow-shrink", (char *)NULL)) {
@@ -2559,14 +2782,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
   case OPT_LIST:
     r = do_list(rbd, io_ctx, lflag, formatter.get());
     if (r < 0) {
-      switch (r) {
-      case -ENOENT:
-        cerr << "rbd: pool " << poolname << " doesn't contain rbd images"
-	     << std::endl;
-        break;
-      default:
-        cerr << "rbd: list: " << cpp_strerror(-r) << std::endl;
-      }
+      cerr << "rbd: list: " << cpp_strerror(-r) << std::endl;
       return -r;
     }
     break;
