@@ -35,8 +35,11 @@ start_rgw=0
 ip=""
 nodaemon=0
 smallmds=0
+hitset=""
 overwrite_conf=1
 cephx=1 #turn cephx on by default
+cache=""
+memstore=0
 
 MON_ADDR=""
 
@@ -61,6 +64,7 @@ usage=$usage"\t-m ip:port\t\tspecify monitor address\n"
 usage=$usage"\t-k keep old configuration files\n"
 usage=$usage"\t-x enable cephx (on by default)\n"
 usage=$usage"\t-X disable cephx\n"
+usage=$usage"\t--hitset <pool> <hit_set_type>: enable hitset tracking\n"
 usage=$usage"\t-o config\t\t add extra config parameters to mds section\n"
 
 usage_exit() {
@@ -142,11 +146,27 @@ case $1 in
     -k )
 	    overwrite_conf=0
 	    ;;
+    --memstore )
+	    memstore=1
+	    ;;
+    --hitset )
+	    hitset="$hitset $2 $3"
+	    shift
+	    shift
+	    ;;
     -o )
 	    extra_conf="$extra_conf	$2
 "
 	    shift
 	    ;;
+    --cache )
+	    if [ -z "$cache" ]; then
+		cache="$2"
+	    else
+		cache="$cache $2"
+	    fi
+	    shift
+	    ;;
     * )
 	    usage_exit
 esac
@@ -221,6 +241,10 @@ if [ -n "$MON_ADDR" ]; then
 	CMDS_ARGS=" -m "$MON_ADDR
 fi
 
+if [ "$memstore" -eq 1 ]; then
+    COSDMEMSTORE='
+	osd objectstore = memstore'
+fi
 
 # lockdep everywhere?
 # export CEPH_ARGS="--lockdep 1"
@@ -298,6 +322,7 @@ if [ "$start_mon" -eq 1 ]; then
 		        cat <<EOF > $conf
 ; generated by vstart.sh on `date`
 [global]
+        fsid = $(uuidgen)
         osd pg bits = 3
         osd pgp bits = 5  ; (invalid, but ceph should cope!)
         osd crush chooseleaf type = 0
@@ -339,6 +364,7 @@ $DAEMONOPTS
         osd scrub load threshold = 5.0
         osd debug op order = true
 $COSDDEBUG
+$COSDMEMSTORE
 $extra_conf
 [mon]
         mon pg warn min per osd = 10
@@ -354,7 +380,7 @@ EOF
 		if [ `echo $IP | grep '^127\\.'` ]
 		then
 			echo
-			echo "WARNING: hostname resolves to loopback; remote hosts will not be able to"
+			echo "NOTE: hostname resolves to loopback; remote hosts will not be able to"
 			echo "  connect.  either adjust /etc/hosts, or edit this script to use your"
 			echo "  machine's real IP."
 			echo
@@ -452,7 +478,7 @@ if [ "$smallmds" -eq 1 ]; then
 EOF
 fi
 
-if [ "$start_mds" -eq 1 ]; then
+if [ "$start_mds" -eq 1 -a "$CEPH_NUM_MDS" -gt 0 ]; then
     mds=0
     for name in a b c d e f g h i j k l m n o p
     do
@@ -494,9 +520,9 @@ EOF
 
 #valgrind --tool=massif $CEPH_BIN/ceph-mds $ARGS --mds_log_max_segments 2 --mds_thrash_fragments 0 --mds_thrash_exports 0 > m  #--debug_ms 20
 #$CEPH_BIN/ceph-mds -d $ARGS --mds_thrash_fragments 0 --mds_thrash_exports 0 #--debug_ms 20
-#$CEPH_ADM mds set_max_mds 2
+#$CEPH_ADM mds set max_mds 2
     done
-    cmd="$CEPH_ADM mds set_max_mds $CEPH_NUM_MDS"
+    cmd="$CEPH_ADM mds set max_mds $CEPH_NUM_MDS"
     echo $cmd
     $cmd
 fi
@@ -520,10 +546,21 @@ $DAEMONOPTS
 EOF
 		    mkdir -p $CEPH_OUT_DIR/htdocs
 		    mkdir -p $CEPH_OUT_DIR/fastcgi_sock
+		    APACHE2_MODULE_PATH="/usr/lib/apache2/modules"
+		    APACHE2_EXTRA_MODULES_NAME="mpm_prefork authz_core"
+		    for module in $APACHE2_EXTRA_MODULES_NAME
+		    do
+			    if [ -f "${APACHE2_MODULE_PATH}/mod_${module}.so" ]; then
+				    APACHE2_EXTRA_MODULES="${APACHE2_EXTRA_MODULES}LoadModule ${module}_module ${APACHE2_MODULE_PATH}/mod_${module}.so
+"
+			    fi 
+		    done
+		    echo $APACHE2_EXTRA_MODULES
 		    cat <<EOF > $CEPH_OUT_DIR/apache.conf
 LoadModule env_module /usr/lib/apache2/modules/mod_env.so
 LoadModule rewrite_module /usr/lib/apache2/modules/mod_rewrite.so
 LoadModule fastcgi_module /usr/lib/apache2/modules/mod_fastcgi.so
+$APACHE2_EXTRA_MODULES
 
 Listen $rgwport
 ServerName rgwtest.example.com
@@ -574,6 +611,39 @@ fi
 
 echo "started.  stop.sh to stop.  see out/* (e.g. 'tail -f out/????') for debug output."
 
+do_cache() {
+    while [ -n "$*" ]; do
+	p="$1"
+	shift
+	echo "creating cache for pool $p ..."
+	$SUDO $CEPH_ADM <<EOF
+osd pool create ${p}-cache 8
+osd tier add $p ${p}-cache
+osd tier cache-mode ${p}-cache writeback
+osd tier set-overlay $p ${p}-cache
+quit
+EOF
+    done
+}
+do_cache $cache
+
+do_hitsets() {
+    while [ -n "$*" ]; do
+	pool="$1"
+	type="$2"
+	shift
+	shift
+	echo "setting hit_set on pool $pool type $type ..."
+	$CEPH_ADM <<EOF
+osd pool set $pool hit_set_type $type
+osd pool set $pool hit_set_count 8
+osd pool set $pool hit_set_period 30
+quit
+EOF
+    done
+}
+do_hitsets $hitset
+
 echo ""
 echo "export PYTHONPATH=./pybind"
 echo "export LD_LIBRARY_PATH=.libs"
